import urllib
import urlparse
import mock

from cubicweb.devtools.httptest import CubicWebServerTC
from cubicweb.devtools.testlib import CubicWebTC
from cubicweb.predicates import yes

from cubes.oauth.entities import ServiceAdapter
from cubes.oauth import views as oauth_views


class FakeRequestRession(object):
    return_value = None

    def __init__(self, base_url):
        self.base_url = base_url

    def get(self, url):
        if url.startswith('me?'):
            if self.return_value:
                m = self.return_value
                self.return_value = None
                return m
            m = mock.Mock()
            m.json.return_value = {
                'id': u'1111',
                'last_name': u'AName',
                'first_name': u'AFName',
                'username': u'zeuser',
                'email': u'zeuser@example.com'}
            return m
        return ''


class FakeRauthService(object):
    tokens = (u'token{}'.format(i) for i in xrange(9999))

    def __init__(
            self, name, client_id, client_secret, authorize_url,
            access_token_url, base_url):
        self.name = name
        self.client_id = client_id
        self.client_secret = client_secret
        self.authorize_url = authorize_url
        self.access_token_url = access_token_url
        self.base_url = base_url

    def get_authorize_url(self, redirect_uri, scope):
        return self.authorize_url + '?' + urllib.urlencode(
            {"redirect_uri": redirect_uri, 'scope': scope})

    def get_auth_session(self, data):
        if data['code'] == u'goodcode':
            s = FakeRequestRession(self.base_url)
            s.access_token = self.tokens.next()
        return s


class FakeServiceAdapter(ServiceAdapter):
    __select__ = ServiceAdapter.__select__ & yes()

    @property
    def oauth2_service(self):
        provider = self.provider
        return FakeRauthService(
            name=self.entity.application_name,
            client_id=self.entity.application_id,
            client_secret=self.entity.application_secret,
            authorize_url=provider.authorize_url,
            access_token_url=provider.access_token_url,
            base_url=provider.base_url,
        )


class OAuthTC(CubicWebServerTC):
    anonymous_allowed = True

    application_name = u'oauth_test'
    application_id = u'123456'
    application_secret = u'secret'

    def setUp(self):
        CubicWebTC.setUp(self)
        self.vreg.register(FakeServiceAdapter)
        with self.admin_access.repo_cnx() as cnx:
            service = cnx.create_entity(
                'ExternalAuthService',
                provider=cnx.find(
                    'ExternalAuthProvider', spid='facebook').one(),
                application_name=self.application_name,
                application_id=self.application_id,
                application_secret=self.application_secret
                )
            self.provider = \
                service.cw_adapt_to('externalauth.service').provider
            cnx.commit()
        oauth_views.NEGOSTATE = {}
        self.start_server()

    def tearDown(self):
        self.vreg.unregister(FakeServiceAdapter)
        super(OAuthTC, self).tearDown()

    def login(self):
        response = self.web_request('?__externalauthprovider=facebook')
        location = response.getheader('location')

        redirect_uri = urlparse.parse_qs(
            location.split('?')[1])['redirect_uri'][0]
        assert redirect_uri.startswith(self.config['base-url'])

        url = redirect_uri + '&code=goodcode'
        url = url[len(self.config['base-url']):]

        return self.web_request(url)

    def test_wrong_providername(self):
        response = self.web_request('/?__externalauthprovider=wrong')
        self.assertEqual(303, response.status)

    def test_1st_redirect(self):
        response = self.web_request('/?__externalauthprovider=facebook')
        location = response.getheader('location')
        expected_location = self.provider.authorize_url + '?' \
            + urllib.urlencode({
                'scope': self.provider.scope,
                'redirect_uri':
                self.config.get('base-url') + 'externalauth-confirm'
                + '?__externalauth_negociationid='
                + oauth_views.NEGOSTATE.keys()[0]})
        self.assertEqual(
            expected_location, location)
        self.assertEqual(303, response.status)

    def test_creation(self):
        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute('Any X WHERE X login "afname.aname"')
            self.assertEqual(0, rset.rowcount)

        response = self.login()
        location = response.getheader('location')
        expected_location = self.config['base-url']
        self.assertEqual(303, response.status)
        self.assertEqual(
            expected_location, location)

        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute('Any X WHERE X login "afname.aname"')
            self.assertEqual(1, rset.rowcount)

    def test_various_creation(self):
        def set_next_me(me):
            FakeRequestRession.return_value = mock.Mock()
            FakeRequestRession.return_value.json.return_value = me

        # with a name
        set_next_me({
            'id': u'1112',
            'name': u'Achille Zavata',
            'last_name': u'AName',
            'first_name': u'AFName',
            'username': u'zeuser',
            'email': u'zeuser@example.com'})

        self.login()
        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute('Any X WHERE X login "achille.zavata"')
            self.assertEqual(1, rset.rowcount)

        # with only a first name
        set_next_me({
            'id': u'1113',
            'first_name': u'AFName',
            'last_name': u'',
            'username': u'zeuser',
            'email': u'zeuser2@example.com'})

        self.login()
        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute('Any X WHERE X login "afname"')
            self.assertEqual(1, rset.rowcount)

        # with only a last name
        set_next_me({
            'id': u'1114',
            'last_name': u'AName',
            'username': u'zeuser',
            'email': u'zeuser3@example.com'})

        self.login()
        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute('Any X WHERE X login "aname"')
            self.assertEqual(1, rset.rowcount)

        # with no name at all
        set_next_me({
            'id': u'1115',
            'name': u'',
            'last_name': None,
            'username': u'zeuser',
            'email': u'zeuser4@example.com'})

        self.login()
        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute('Any X WHERE X login "zeuser"')
            self.assertEqual(1, rset.rowcount)

        # with no name nor username
        set_next_me({
            'id': u'1116',
            'name': u'',
            'last_name': None,
            'username': u'',
            'email': u'zeuser5@example.com'})

        self.login()
        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute('Any X WHERE X login "zeuser5"')
            self.assertEqual(1, rset.rowcount)

        # with nothing but an id
        set_next_me({
            'id': u'1117'})

        self.login()
        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute('Any X WHERE X login "1117"')
            self.assertEqual(1, rset.rowcount)

    def test_login(self):
        with self.admin_access.repo_cnx() as cnx:
            user = cnx.create_entity(
                'CWUser', login=u"zeuser", upassword='',
                in_group=cnx.find('CWGroup', name='users').one())
            cnx.create_entity(
                'ExternalIdentity',
                identity_of=user, provider=cnx.find(
                    'ExternalAuthProvider', spid='facebook').one(),
                uid=u'1111')
            rset = cnx.execute('Any X WHERE X login "zeuser"')
            self.assertEqual(1, rset.rowcount)
            cnx.commit()

        response = self.login()
        location = response.getheader('location')
        expected_location = self.config['base-url']
        self.assertEqual(303, response.status)
        self.assertEqual(expected_location, location)

        cookie = dict(
            token.strip().split('=') for token in
            response.getheader('Set-Cookie').split(';'))
        assert cookie['__data_session'].startswith("zeuser")

        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute('Any X WHERE X login "zeuser"')
            self.assertEqual(1, rset.rowcount)



# -*- coding: utf-8 -*-
#
# copyright 2013 Unlish (Montpellier, FRANCE), all rights reserved.
# contact http://www.unlish.com -- mailto:contact@unlish.com
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-oauth schema."""

from yams.buildobjs import (
    String, EntityType, SubjectRelation, Datetime, Boolean
)

from cubicweb.schema import ERQLExpression


# TODO set the maxsize of the String attributes.

sp_attrs_perms = {
    'read': ('managers', 'users', 'guests'),
    'update': ('managers',),
}


class ExternalAuthProvider(EntityType):
    __permissions__ = {
        'read': ('managers', 'users', 'guests'),
        'add': ('managers',),
        'update': ('managers',),
        'delete': ('managers',)
    }
    spid = String(
        __permissions__=sp_attrs_perms,
        maxsize=16, required=True, unique=True,
        description="Provider unique id ('facebook', 'twitter')")
    name = String(
        __permissions__=sp_attrs_perms,
        maxsize=16, required=True, unique=True,
        description="Provider user-friendly name ('Facebook', 'Twitter')")


# XXX make it workfloable
class ExternalAuthService(EntityType):
    __permissions__ = {
        'read': ('managers',),
        'add': ('managers',),
        'update': ('managers',),
        'delete': ('managers',),
    }
    __unique_together__ = [
        ('provider', 'application_id'),
    ]
    provider = SubjectRelation(
        'ExternalAuthProvider',
        __permissions__={
            'read': ('managers',),
            'add': ('managers',),
            'delete': ('managers',)
        },
        cardinality='1*',
        inlined=True,
        composite='object'
    )
    application_name = String(
        maxsize=64, description="Application name",
        __permissions__=sp_attrs_perms)
    application_id = String(
        maxsize=64, description="OAuth2 client_id.",
        __permissions__=sp_attrs_perms)
    application_secret = String(
        maxsize=64, description="OAuth2 client_secret",
        __permissions__=sp_attrs_perms)


class ExternalIdentity(EntityType):
    __permissions__ = {
        'read':   ('managers', ERQLExpression('X identity_of U')),
        'add':    ('managers',),
        'update': ('managers',),
        'delete': ('managers', ERQLExpression('X identity_of U'))
    }
    __unique_together__ = [('provider', 'uid')]
    provider = SubjectRelation(
        'ExternalAuthProvider', cardinality='1*', inlined=True,
        __permissions__={
            'read': ('managers', 'users'),
            'add': ('managers',),
            'delete': ('managers',)
        },
        composite='object'
    )
    identity_of = SubjectRelation(
        'CWUser', cardinality='?*', inlined=True,
        __permissions__={
            'read':   ('managers', 'users',),
            'add':    ('managers',),
            'delete': ('managers',)
        },
        composite='object'
    )
    uid = String(
        required=True,
        __permissions__={
            'read':   ('managers', 'users',),
            'update': ('managers',),
        }
    )


class OAuth2Session(EntityType):
    __permissions__ = {
        'read':   ('managers', ERQLExpression('X external_identity I, I identity_of U')),
        'add':    ('managers',),
        'update': ('managers',),
        'delete': ('managers', ERQLExpression('X external_identity I, I identity_of U'))
    }

    external_identity = SubjectRelation(
        'ExternalIdentity', cardinality='1*', inlined=True,
        __permissions__={
            'read':   ('managers', 'users',),
            'add':    ('managers',),
            'delete': ('managers',)
        }
    )
    service = SubjectRelation(
        'ExternalAuthService', cardinality='1*', inlined=True,
        __permissions__={
            'read':   ('managers', 'users',),
            'add':    ('managers',),
            'delete': ('managers',)
        },
        composite='object'
    )
    access_token = String(required=True, unique=True)
    active = Boolean(required=True, default=True)
    expiry = Datetime()



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-oauth entity's classes"""
from rauth import OAuth2Service

from cubicweb.predicates import is_instance
from cubicweb.appobject import AppObject
from cubicweb.view import EntityAdapter


class Provider(AppObject):
    __registry__ = 'provider'
    __abstract__ = True


class FacebookProvider(Provider):
    __regid__ = 'facebook'
    authorize_url = 'https://graph.facebook.com/oauth/authorize'
    access_token_url = 'https://graph.facebook.com/oauth/access_token'
    base_url = 'https://graph.facebook.com/'
    scope = 'email'

    def retrieve_info(self, request):
        me = request.get(
            'me?fields=name,first_name,last_name,email,username'
        ).json()
        self.debug('Retrieved info %s', me)
        return {
            'uid': me['id'],
            'email': me.get('email'),
            'name': me.get('name'),
            'firstname': me.get('first_name'),
            'lastname': me.get('last_name'),
            'username': me.get('username')
        }


class ServiceAdapter(EntityAdapter):
    __regid__ = 'externalauth.service'
    __select__ = is_instance('ExternalAuthService')

    @property
    def provider(self):
        spid = self.entity.provider[0].spid
        return self._cw.vreg['provider'][spid][0](self._cw)

    @property
    def oauth2_service(self):
        provider = self.provider
        return OAuth2Service(
            name=self.entity.application_name,
            client_id=self.entity.application_id,
            client_secret=self.entity.application_secret,
            authorize_url=provider.authorize_url,
            access_token_url=provider.access_token_url,
            base_url=provider.base_url,
        )



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-oauth views/forms/actions/components for web ui"""

import re
import urllib
import urlparse
import unicodedata

from cubicweb.utils import make_uid
from cubicweb.predicates import (
    anonymous_user, configuration_values, match_form_params)

from cubicweb.view import View
from cubicweb.web.controller import Controller
from cubicweb.web.views import authentication
from cubicweb.web.views import basecomponents, basetemplates
from cubicweb.web import Redirect, formfields, formwidgets

from cubicweb.web import InvalidSession

from cubes.oauth.authplugin import EXT_TOKEN

_ = unicode

# XXX replace with multi-instance safe mechanism
NEGOSTATE = {}  # negoid -> service coordinates
COLLECT_LIST = []

login_re = re.compile('(?P<base>.*)(\.(?P<index>\d+))?')


def get_next_login(login):
    m = login_re.match(login)
    assert m is not None
    if m.group('index'):
        index = int(m.group('index')) + 1
    else:
        index = 1
    return '{}.{}'.format(login, index)


class ExternalAuthConfirm(Controller):
    __regid__ = 'externalauth-confirm'
    __select__ = match_form_params('code', '__externalauth_negociationid')

    def publish(self, rset=None):
        form = self._cw.form
        negociationid = form.get('__externalauth_negociationid')

        if not negociationid:
            raise Redirect(self._cw.build_url())

        nego = NEGOSTATE.pop(negociationid, None)

        if nego is None:
            raise Redirect(self._cw.build_url())

        # XXX If the user was created just now, optionnaly redirect to its
        # profile instead of the final_redirect
        raise Redirect(self._cw.build_url(nego['final_redirect']))


class ExternalAuthError(Controller):
    __regid__ = 'externalauth-confirm'
    __select__ = match_form_params(
        'code_error', 'code_message', '__externalauth_negociationid')

    def publish(self, rset=None):
        form = self._cw.form

        raise Redirect(self._cw.build_url(
            __message='error while authenticating: %s %s'
            % (form.get('error_code'), form.get('error_message', ''))))


class yes_marker(object):
    def __bool__(self):
        return True


class ExternalAuthRetrieverStart(authentication.WebAuthInfoRetriever):
    """External authentication first step.

    Will only redirect on the demanded service.
    """
    __regid__ = 'externalauth-authenticate-start'
    order = 9

    def request_has_auth_info(self, req):
        return '__externalauthprovider' in req.form

    def revalidate_login(self, req):
        # Always invalidate the current session
        raise InvalidSession()

    def authentication_information(self, req):
        providername = req.form.get('__externalauthprovider')
        if providername is None:
            raise authentication.NoAuthInfo

        self.info(
            'external authenticator (%s) building auth info %s',
            self.__class__.__name__, req.form
        )

        vreg = self._cw  # YES, see cw/web/views/authentication.py ~106
        repo = vreg.config.repository(vreg)
        with repo.internal_cnx() as cnx:
            servicerset = cnx.execute(
                'ExternalAuthService S WHERE S provider P, P spid %(spid)s',
                {'spid': providername}
            )
            if not servicerset:
                self.error('no service for provider: %s' % providername)
                raise Redirect(req.build_url(
                    __message='no service for provider %s' % providername
                ))

            service = servicerset.one()

            path = req.relative_path(includeparams=True)
            # Remove the 'vid' parameter if the selected view is 'loggedout'
            if '?' in path:
                path, query = path.split('?', 1)
                params = urlparse.parse_qs(query)
                if params.get('vid') == ['loggedout']:
                    del params['vid']
                params.pop('__externalauthprovider', None)
                if params:
                    path = '?'.join(
                        (path, urllib.urlencode(params, doseq=True)))

            negoid = make_uid('nego')
            nego = {'service': service.eid,
                    'redirect_uri': req.build_url(
                        'externalauth-confirm',
                        __externalauth_negociationid=negoid
                    ),
                    'final_redirect': path}
            NEGOSTATE[negoid] = nego
            service_adapter = service.cw_adapt_to('externalauth.service')
            url = service_adapter.oauth2_service.get_authorize_url(
                redirect_uri=nego['redirect_uri'],
                scope=service_adapter.provider.scope
            )
            raise Redirect(url)


class ExternalAuthRetrieverFinish(authentication.WebAuthInfoRetriever):
    __regid__ = 'externalauth-success'
    order = ExternalAuthRetrieverStart.order - 1

    def request_has_auth_info(self, req):
        return (
            '__externalauth_negociationid' in req.form
            and 'code' in req.form)

    def revalidate_login(self, req):
        # Always invalidate the current session
        raise InvalidSession()

    def authentication_information(self, req):
        if not self.request_has_auth_info(req):
            raise authentication.NoAuthInfo

        self.info('external authenticator (%s) building auth info %s',
                  self.__class__.__name__, req.form)

        code = req.form.get('code')
        negociationid = req.form.get('__externalauth_negociationid')

        if not (code and negociationid):
            raise authentication.NoAuthInfo("Invalid oauth code/negociationid")

        nego = NEGOSTATE.get(negociationid, None)

        if nego is None:
            raise authentication.NoAuthInfo(req._('Authentication timeout.'))

        vreg = self._cw  # YES, see cw/web/views/authentication.py ~106
        repo = vreg.config.repository(vreg)
        with repo.internal_cnx() as cnx:
            try:
                service = cnx.entity_from_eid(nego['service'])
                adapter = service.cw_adapt_to('externalauth.service')
                oauth2_session = adapter.oauth2_service.get_auth_session(data={
                    'code': code,
                    'redirect_uri': nego['redirect_uri']
                })

                infos = adapter.provider.retrieve_info(oauth2_session)
                extid_rset = cnx.execute(
                    'ExternalIdentity SI WHERE SI provider P, '
                    'P eid %(peid)s, SI uid %(uid)s',
                    {'peid': service.provider[0].eid, 'uid': infos['uid']}
                )
                assert extid_rset.rowcount < 2
                if extid_rset:
                    external_identity = extid_rset.get_entity(0, 0)
                else:
                    external_identity = cnx.create_entity(
                        'ExternalIdentity', provider=service.provider[0],
                        uid=infos['uid']
                    )

                # deactivate any previous access_token
                cnx.execute(
                    'SET X active FALSE WHERE '
                    'X is OAuth2Session, '
                    'S external_identity SI, SI eid %(sieid)s, '
                    'S service SE, SE eid %(seid)s',
                    {'sieid': external_identity.eid, 'seid': service.eid}
                )
                oauth2_session = cnx.create_entity(
                    'OAuth2Session', service=service,
                    external_identity=external_identity,
                    access_token=oauth2_session.access_token,
                    active=True
                )

                cnx.commit(free_cnxset=False)
            except Exception, e:
                self.exception("Cannot get the oauth2_session")
                raise authentication.NoAuthInfo(str(e))

            if external_identity.identity_of:
                user = external_identity.identity_of[0]
                self.debug('identified %s on %s as %s' % (
                    user.login, service.provider[0].name, infos['uid']))
                login = user.login
            else:
                self.debug(
                    "Unknown identity %s on %s: will create a local account",
                    infos['uid'], service.provider[0].name)

                login = self.forge_login(cnx, infos)
                password = make_uid()

                # TODO insert more informations from the external service
                # XXX We should loop around the call_service and retry with a
                # different login (w. get_next_login) when it raises a
                # ValidationError _or_ if the commit() raise a duplicate key
                # error.

                try:
                    cnx.call_service(
                        'register_user',
                        login=login,
                        password=password,
                        email=infos.get('email', None),
                        firstname=infos.get('firstname', u''),
                        surname=infos.get('lastname', u''))

                    cnx.execute(
                        'SET X identity_of U '
                        'WHERE X eid %(sieid)s, '
                        'U is CWUser, U login %(login)s',
                        {'sieid': external_identity.eid,
                         'login': login}
                    )
                    cnx.commit()
                except:
                    self.exception("Cannot create user %s" % login)
                    raise

        return login, {'__externalauth_directauth': EXT_TOKEN}

    def make_unique_login(self, cnx, login):
        """Return a login that does not yet exists in the database,
        based on the given login"""
        q = 'Any X WHERE X is CWUser, X login %(login)s'
        while True:
            rset = cnx.execute(q, {'login': login})
            if not rset.rowcount:
                break
            login = get_next_login(login)
        return login

    def normalize_login(self, login):
        """Return a string suitable to be a login, based on the given login"""
        login = unicodedata.normalize('NFKD', login)
        login = login.encode('ascii', 'ignore')
        login = login.replace(' ', '.').lower()
        return unicode(login)

    def forge_login(self, cnx, infos):
        """Forge a login based on infos comming from the oauth2 provider.
        """
        if infos.get('name'):
            login = infos['name']
        elif infos.get('firstname') or infos.get('lastname'):
            login = u'{}.{}'.format(
                infos.get('firstname') or u'',
                infos.get('lastname') or u'').strip('.')
        elif infos.get('username'):
            login = infos.get('username')
        elif infos.get('email'):
            login = infos['email'].split('@', 1)[0]
        else:
            login = infos['uid']

        login = self.normalize_login(login)

        return self.make_unique_login(cnx, login)


# form, link and view

class ExternalAuthLogForm(basetemplates.BaseLogForm):
    __regid__ = domid = 'externalauthlogform'
    boxid = 'externalauthlogbox'

    action = ''

    __externalauthprovider = formfields.StringField(
        '__externalauthprovider', label=_('externalauthprovider'),
        widget=formwidgets.TextInput({'class': 'data'}),
        # NOTE: could be a dropdown with a pre-cooked list ...
        value='facebook'
    )

    onclick_args = ('externalauthlogbox', '__externalauthprovider')


class ExternalAuthLoginLink(basecomponents.HeaderComponent):
    __regid__ = 'externalauthlink'
    __select__ = (
        basecomponents.HeaderComponent.__select__ and
        configuration_values('auth-mode', 'cookie') &
        anonymous_user()
    )

    context = _('header-right')
    onclick = "javascript: cw.htmlhelpers.popupLoginBox('%s', '%s');"

    def render(self, w):
        text = self._cw._('external auth login')
        w(u"""<a href="#" onclick="%s">%s</a>""" % (
            self.onclick % ('externalauthlogbox', '__externalauthprovider'),
            text
        ))
        self._cw.view(
            'externalauthlogform', rset=self.cw_rset,
            id='externalauthlogbox', w=w
        )


class ExternalAuthLogFormView(View):
    __regid__ = 'externalauthlogform'
    __select__ = configuration_values('auth-mode', 'cookie')

    help_msg = _("""
Select the provider you want to use to authenticate.
    """)

    title = _('log in using a provider')

    def call(self, id='externalauthlogbox'):
        w = self.w

        w(u'<div id="%s" class="popupLoginBox hidden">' % id)
        w(u'<p>%s</p>' % self._cw._(self.help_msg))
        form = self._cw.vreg['forms'].select('externalauthlogform', self._cw)
        form.render(w=w)
        w(u'</div>')



# -*- coding: utf-8 -*-
#
# copyright 2013 Unlish (Montpellier, FRANCE), all rights reserved.
# contact http://www.unlish.com -- mailto:contact@unlish.com
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-oauth specific hooks and operations."""

from cubicweb.server import hook

from cubes.oauth.authplugin import DirectAuthentifier


class ServerStartupHook(hook.Hook):
    """register authentifier at startup."""

    __regid__ = 'oauth-authentifier-register'
    events = ('server_startup',)
    _negociation_timeout = 300

    def __call__(self):
        self.debug('registering externalauth authentifier')
        self.repo.system_source.add_authentifier(DirectAuthentifier())


        def cleanup_negostates():
            import cubes.oauth.views as oviews

            for negoid in oviews.COLLECT_LIST:
                self.debug('collecting negostate', negoid)
                oviews.NEGOSTATE.pop(negoid, None)
            oviews.COLLECT_LIST[:] = []

            for negoid in oviews.NEGOSTATE:
                oviews.COLLECT_LIST.append(negoid)

        self.repo.looping_task(self._negociation_timeout,
                               cleanup_negostates)
